from viggocore.common.subsystem.driver import Driver
from viggocore.common.subsystem import operation
from viggocore.common import exception


class Manager(object):

    def __init__(self, driver: Driver) -> None:
        self.driver = driver

        self.create = operation.Create(self)
        self.get = operation.Get(self)
        self.list = operation.List(self)
        self.update = operation.Update(self)
        self.delete = operation.Delete(self)
        # NOTE(samueldmq): what do we use this for ?
        self.count = operation.Count(self)

    def init_query(self, session, order_by, resource):
        raise exception.BadRequest(
            f'Method _init_query() not implemented for {resource.__name__}.')

    def valid_dinamic_order_by(self, order_by):
        result = False
        count_points = order_by.count('.')
        if ('.' in order_by and count_points == 1) or count_points == 0:
            result = True
        elif count_points > 1:
            raise exception.BadRequest(
                'order_by item cannot have more than one point.')
        return result



import flask
import uuid

from datetime import datetime
from sqlalchemy import func
from viggocore.common import exception
from viggocore.common.subsystem import operation, manager
from viggocore.subsystem.timeline_event.resource \
    import TimelineEvent, TimelineEventUser


class Create(operation.Create):

    def pre(self, session, **kwargs):
        if 'id' not in kwargs:
            kwargs['id'] = uuid.uuid4().hex
        if 'created_at' not in kwargs:
            kwargs['created_at'] = datetime.now()
        if flask.has_request_context():
            token_id = flask.request.headers.get('token')
            if token_id is not None:
                self.token = self.manager.api.tokens().get(id=token_id)
                kwargs['created_by'] = self.token.user_id
                kwargs['event_by'] = self.token.user_id
        else:
            kwargs['event_by'] = kwargs['created_by']

        kwargs['lat'] = kwargs.get('lat', '0')
        kwargs['lon'] = kwargs.get('lon', '0')
        kwargs['event_at'] = kwargs.get('event_at', datetime.now())

        self.entity = self.driver.instantiate(**kwargs)

        return self.entity.is_stable()


class List(operation.List):

    # TODO passar para o driver do viggocore
    def __filter_params(self, resource, query, **kwargs):
        for k, v in kwargs.items():
            if hasattr(resource, k):
                if isinstance(v, str) and '%' in v:
                    normalize = func.viggocore_normalize
                    query = query.filter(normalize(getattr(resource, k))
                                         .ilike(normalize(v)))
                else:
                    query = query.filter(getattr(resource, k) == v)
        return query

    def pre(self, **kwargs):
        self.user_id = kwargs.get('user_id', None)
        if not self.user_id:
            raise exception.BadRequest('Erro! user_id is empty')
        return super().pre(**kwargs)

    def do(self, session, **kwargs):
        timeline_events = []

        timeline_events_query = session. \
            query(TimelineEvent). \
            join(TimelineEventUser,
                 TimelineEventUser.timeline_event_id == TimelineEvent.id). \
            filter(TimelineEventUser.user_id == self.user_id)

        timeline_events_query = self.__filter_params(
            TimelineEvent, timeline_events_query, **kwargs)

        timeline_events = timeline_events_query.distinct(). \
            order_by(TimelineEvent.created_at.desc()). \
            limit(TimelineEvent.LIMIT_SEARCH). \
            all()

        return timeline_events


class Manager(manager.Manager):

    def __init__(self, driver) -> None:
        super().__init__(driver)
        self.create = Create(self)
        self.list = List(self)



import json

import flask

from viggocore.common import exception, utils
from viggocore.common.exception import BadRequest
from viggocore.common.subsystem import controller
from viggocore.subsystem.image.resource import QualityImage
from datetime import datetime
import datetime as datetime2
from viggocore.common.subsystem import entity


class Controller(controller.Controller):

    def __init__(self, manager, resource_wrap, collection_wrap):
        super(Controller, self).__init__(
            manager, resource_wrap, collection_wrap)

    def _get_name_in_args(self):
        name = flask.request.args.get('name', None)
        if not name:
            raise BadRequest()
        return name

    def domain_by_name(self):
        try:
            name = self._get_name_in_args()
            domain = self.manager.domain_by_name(domain_name=name)
            domain_dict = domain.to_dict()
            if 'settings' in domain_dict.keys():
                domain_dict.pop('settings')
            response = {self.resource_wrap: domain_dict}
            return flask.Response(response=json.dumps(response, default=str),
                                  status=200,
                                  mimetype="application/json")
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

    def domain_logo_by_name(self):
        try:
            kwargs = {}
            quality = flask.request.args.get('quality', None)
            kwargs['quality'] = \
                QualityImage[quality] if quality else QualityImage.med
            name = self._get_name_in_args()
            folder, filename = self.manager.domain_logo_by_name(
                domain_name=name, **kwargs)
            return flask.send_from_directory(folder, filename)
        except KeyError:
            return self.get_bad_request('Unknown Quality')
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

    def get_token_id(self):
        return flask.request.headers.get('token')

    def get_token(self, token_id):
        return self.manager.api.tokens().get(id=token_id)

    def get_domain(self, domain_id):
        return self.manager.api.domains().get(id=domain_id)

    def get_domain_id_from_token(self, token):
        user = self.manager.api.users().get(id=token.user_id)
        return user.domain_id

    def get_domain_id(self):
        token = self.get_token(self.get_token_id())
        domain_id = self.get_domain_id_from_token(token)
        return domain_id

    def upload_logo(self, id):
        try:
            token = flask.request.headers.get('token')
            file = flask.request.files.get('file', None)
            if not file:
                raise exception.BadRequest('ERROR! File not found in request.')

            image = self.manager.upload_logo(id=id, token=token, file=file)

            response = {'image': image.to_dict()}
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        return flask.Response(response=utils.to_json(response),
                              status=201,
                              mimetype="application/json")

    def remove_logo(self, id):
        try:
            self.manager.remove_logo(id=id)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        return flask.Response(response=None,
                              status=204,
                              mimetype="application/json")

    def register(self):
        try:
            data = flask.request.get_json()

            username = data.get('username', 'admin').lower()
            email = data.get('email', None)
            password = data.get('password', None)
            domain_name = data.get('domain_name', None)
            domain_display_name = data.get('domain_display_name', None)
            application_name = data.get('application_name', None)

            self.manager.register(
                username=username, email=email,
                password=password, domain_name=domain_name,
                domain_display_name=domain_display_name,
                application_name=application_name)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        return flask.Response(response=None,
                              status=204,
                              mimetype="application/json")

    def activate(self, id1, id2):
        try:
            token_id = flask.request.headers.get('token')
            domain = self.manager.activate(
                token_id=token_id, domain_id=id1, user_admin_id=id2)

            response = {'domain': domain.to_dict()}
            return flask.Response(response=utils.to_json(response),
                                  status=200,
                                  mimetype="application/json")
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

    def update_settings(self, id):
        try:
            data = flask.request.get_json()

            settings = self.manager.update_settings(id=id, **data)
            response = {'settings': settings}

            return flask.Response(response=utils.to_json(response),
                                  status=200,
                                  mimetype="application/json")
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

    def _get_keys_from_args(self):
        keys = flask.request.args.get('keys')
        if not keys:
            raise exception.BadRequest(
                'ERRO! The keys parameter was not passed correctly')
        return list(filter(None, keys.split(',')))

    def remove_settings(self, id):
        try:
            keys = self._get_keys_from_args()
            kwargs = {'keys': keys}

            settings = self.manager.remove_settings(id=id, **kwargs)
            response = {'settings': settings}

            return flask.Response(response=utils.to_json(response),
                                  status=200,
                                  mimetype="application/json")
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

    def get_domain_settings_by_keys(self, id):
        try:
            keys = self._get_keys_from_args()
            kwargs = {'keys': keys}

            settings = self.manager.get_domain_settings_by_keys(
                id=id, **kwargs)
            response = {'id': id, 'settings': settings}

            return flask.Response(response=utils.to_json(response),
                                  status=200,
                                  mimetype="application/json")
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

    def send_email_activate_account(self):
        try:
            data = flask.request.get_json()

            self.manager.send_email_activate_account(**data)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        return flask.Response(response=None,
                              status=204,
                              mimetype="application/json")

    def get_domain_settings_by_publicas(self, id):
        try:
            kwargs = {'keys': ['public']}

            settings = self.manager.get_domain_settings_by_keys(
                id=id, **kwargs)
            response = {'id': id, 'settings': settings}

            return flask.Response(response=utils.to_json(response),
                                  status=200,
                                  mimetype="application/json")
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

    def get_files_size(self, id):
        try:
            files_size = self.manager.get_files_size(id=id)
            response = {'id': id, 'sizes': files_size}

            return flask.Response(response=utils.to_json(response),
                                  status=200,
                                  mimetype="application/json")
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

    # return start and end
    def _get_page_and_page_size(self, page, page_size):
        if page < 0:
            raise exception.BadRequest(
                'the page parameter must be greater than -1')
        if page_size < 1:
            raise exception.BadRequest(
                'the page_size parameter must be greater than 0')

        if page == 0:
            return (page, page_size)
        else:
            start = page * page_size
            end = start + page_size
            return (start, end)

    def _convert_de_ate(self, **kwargs):
        de = kwargs.pop('de', '2020-01-01-0300')
        ate = kwargs.pop('ate', '9020-01-01-0300')

        if de and ate:
            try:
                inicio = datetime.strptime(de.replace(' ', '+'), '%Y-%m-%d%z')
                fim = datetime.strptime(ate.replace(' ', '+'), '%Y-%m-%d%z') +\
                    datetime2.timedelta(days=1)

            except Exception:
                inicio = datetime.strptime(de, entity.DATE_FMT)
                fim = datetime.strptime(ate, entity.DATE_FMT) +\
                    datetime2.timedelta(days=1)
            kwargs['de'] = inicio
            kwargs['ate'] = fim

        return kwargs

    def get_usage_info_by_domain(self):
        filters = self._filters_parse()
        filters = self._filters_cleanup(filters)

        try:
            filters = self._parse_list_options(filters)
            filters = self._convert_de_ate(**filters)

            require_pagination = filters.pop('require_pagination', False)
            page = filters.pop('page', None)
            page_size = filters.pop('page_size', None)
            domain_name = filters.pop('domain_name', None)
            active = filters.pop('active', None)

            entities, totals = self.manager.get_usage_info_by_domain(**filters)

        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)
        except ValueError:
            raise exception.BadRequest('page or page_size is invalid')

        if domain_name is not None:
            if '%' in domain_name:
                domain_name = domain_name.replace('%', '')
                entities = list(
                    filter(lambda x: domain_name in x['domain_name'],
                           entities))
            else:
                entities = list(
                    filter(lambda x: domain_name == x['domain_name'],
                           entities))

        if active is not None:
            entities = list(
                filter(lambda x: x['active'] == active, entities))

        entities = sorted(entities, key=lambda item: item['domain_name'])

        total = len(entities)

        if require_pagination and None not in [page, page_size]:
            page = int(page)
            page_size = int(page_size)

            start, end = self._get_page_and_page_size(page, page_size)

            response = {self.collection_wrap: entities[start:end],
                        'totals': totals}

            response.update({'pagination': {'page': page,
                                            'page_size': page_size,
                                            'total': total}})
        else:
            response = {self.collection_wrap: entities, 'totals': totals}

        return flask.Response(response=utils.to_json(response),
                              status=200,
                              mimetype="application/json")



from sqlalchemy import func, or_, and_
from viggocore.subsystem.application.resource import Application
from typing import List
from viggocore.common import exception
from viggocore.common.input import RouteResource, InputResource, \
    InputResourceUtils
from viggocore.common.subsystem import operation, manager
from viggocore.subsystem.capability.resource import Capability
from viggocore.subsystem.policy.resource import Policy
from viggocore.subsystem.role.resource import Role
from viggocore.subsystem.route.resource import Route
from viggocore.common.subsystem.pagination import Pagination
from sqlalchemy.sql import text


class Create(operation.Create):

    def pre(self, session, **kwargs) -> bool:
        self.exceptions = kwargs.pop('exceptions', [])

        return super().pre(session, **kwargs)

    def do(self, session, **kwargs):
        super().do(session)
        self.manager.create_user_capabilities_and_policies(id=self.entity.id,
                                                           session=session)
        if self.entity.name != Application.DEFAULT:
            self.manager.create_admin_capabilities_and_policies(
                id=self.entity.id, session=session, exceptions=self.exceptions)
        return self.entity


class ListManager(operation.List):

    def apply_filters(self, query, resource, **kwargs):
        for k, v in kwargs.items():
            if hasattr(resource, k):
                if k == 'tag':
                    values = v
                    if len(v) > 0 and v[0] == '#':
                        values = v[1:]
                    values = values.split(',')
                    filter_tags = []
                    for value in values:
                        filter_tags.append(
                            getattr(resource, k)
                            .like('%#' + str(value) + ' %'))
                    query = query.filter(or_(*filter_tags))
                elif k == 'tag_name':
                    values = v
                    if len(v) > 0 and v[0] == '#':
                        values = v[1:]
                    filter_tags = []
                    query = query.filter(
                        getattr(resource, k) == '#' + str(values) + ' ')
                elif isinstance(v, str) and '%' in v:
                    normalize = func.viggocore_normalize
                    query = query.filter(normalize(getattr(resource, k))
                                         .ilike(normalize(v)))
                else:
                    query = query.filter(getattr(resource, k) == v)

        return query

    def apply_pagination(self, query, pagination: Pagination):
        if (pagination.order_by is not None and pagination.page is not None
           and pagination.page_size is not None):
            query = query.order_by(text(pagination.order_by))

        if pagination.page_size is not None:
            query = query.limit(pagination.page_size)
            if pagination.page is not None:
                query = query.offset(pagination.page * pagination.page_size)

        return query

    def do(self, session, **kwargs):
        not_default = kwargs.pop('not_default', False)
        query = session.query(Application)

        pagination = Pagination.get_pagination(Application, **kwargs)

        if not_default is True:
            query = query.filter(Application.name != Application.DEFAULT)

        query = self.apply_filters(query, Application, **kwargs)
        query = self.apply_pagination(query, pagination)

        result = query.all()
        return result


class CreateUserCapabilitiesAndPolicies(operation.Operation):

    def pre(self, session, id, **kwargs) -> bool:
        self.application_id = id
        self.user_resources = self.manager.bootstrap_resources.USER
        self.role_id = self.manager.api.roles().\
            get_role_by_name(role_name=Role.USER).id

        return True

    def do(self, session, **kwargs):
        self.resources = {'resources': self.user_resources}
        self.manager.api.capabilities().create_capabilities(
            id=self.application_id, **self.resources)

        self.resources['application_id'] = self.application_id
        self.manager.api.roles().create_policies(id=self.role_id,
                                                 **self.resources)


class CreateAdminCapabilitiesAndPolicies(operation.Operation):

    def _map_routes(self, routes: List[Route]) -> List[RouteResource]:
        resources = [(route.url, route.method) for route in routes]
        return resources

    def _filter_resources(self, all_resources: List[RouteResource],
                          exceptions_resources: List[RouteResource],
                          sysadmin_exclusive_resources: List[RouteResource],
                          user_resources: List[RouteResource]) \
            -> List[RouteResource]:
        resources = InputResourceUtils.diff_resources(
            all_resources, sysadmin_exclusive_resources, user_resources,
            exceptions_resources)
        return resources

    def pre(self, session, id, exceptions: List[InputResource] = [], **kwargs):
        self.application_id = id
        exceptions_resources = InputResourceUtils.parse_resources(exceptions)

        routes = self.manager.api.routes().list(active=True)
        routes_resources = self._map_routes(routes)

        self.admin_role_id = self.manager.api.roles().\
            get_role_by_name(role_name=Role.ADMIN).id

        self.admin_resources = self._filter_resources(
            routes_resources, exceptions_resources,
            self.manager.bootstrap_resources.SYSADMIN_EXCLUSIVE,
            self.manager.bootstrap_resources.USER)
        return True

    def do(self, session, **kwargs):
        self.resources = {'resources': self.admin_resources}
        self.manager.api.capabilities().create_capabilities(
            id=self.application_id, **self.resources)

        self.resources['application_id'] = self.application_id
        self.manager.api.roles().create_policies(id=self.admin_role_id,
                                                 **self.resources)


class CreateCapabilitiesWithExceptions(operation.Operation):

    def _filter_resources(self, routes: List[Route],
                          exceptions: List[RouteResource],
                          sysadmin_exclusive_resources: List[RouteResource],
                          user_resources: List[RouteResource]) \
            -> List[RouteResource]:
        all_resources = [(route.url, route.method) for route in routes]
        resources = InputResourceUtils.diff_resources(
            all_resources, sysadmin_exclusive_resources, user_resources,
            exceptions)
        return resources

    def pre(self, session, id: str, **kwargs):
        self.application_id = id
        exceptions = kwargs.get('exceptions', None)

        if not self.application_id or exceptions is None:
            raise exception.BadRequest()

        routes = self.manager.api.routes().list(active=True)
        exceptions_resources = InputResourceUtils.parse_resources(exceptions)
        self.resources = self.\
            _filter_resources(routes,
                              exceptions_resources,
                              self.manager.bootstrap_resoures.
                              SYSADMIN_EXCLUSIVE,
                              self.manager.bootstrap_resources.USER)

        return self.driver.get(id, session=session) is not None

    def do(self, session, **kwargs):
        data = {'resources', self.resources}
        self.manager.api.capabilities().\
            create_capabilities(id=self.application_id, **data)


class GetRoles(operation.Operation):

    def pre(self, session, id, **kwargs):
        self.application_id = id
        return self.driver.get(id, session=session) is not None

    def do(self, session, **kwargs):
        roles = session.query(Role). \
            join(Policy). \
            join(Capability). \
            filter(and_(Capability.application_id == self.application_id,
                        Role.name != Role.USER)). \
            distinct()
        return roles


class UpdateSettings(operation.Update):

    def pre(self, session, id: str, **kwargs) -> bool:
        self.settings = kwargs
        if self.settings is None or not self.settings:
            raise exception.BadRequest("Erro! There is not a setting")
        return super().pre(session=session, id=id)

    def do(self, session, **kwargs):
        result = {}
        for key, value in self.settings.items():
            new_value = self.entity.update_setting(key, value)
            result[key] = new_value
        super().do(session)

        return result


class RemoveSettings(operation.Update):

    def pre(self, session, id: str, **kwargs) -> bool:
        self.keys = kwargs.get('keys', [])
        if not self.keys:
            raise exception.BadRequest('Erro! keys are empty')
        super().pre(session, id=id)

        return self.entity.is_stable()

    def do(self, session, **kwargs):
        result = {}
        for key in self.keys:
            value = self.entity.remove_setting(key)
            result[key] = value
        super().do(session=session)

        return result


class GetApplicationSettingsByKeys(operation.Get):

    def pre(self, session, id, **kwargs):
        self.keys = kwargs.get('keys', [])
        if not self.keys:
            raise exception.BadRequest('Erro! keys are empty')
        return super().pre(session, id=id)

    def do(self, session, **kwargs):
        entity = super().do(session=session)
        settings = {}
        for key in self.keys:
            value = entity.settings.get(key, None)
            if value is not None:
                settings[key] = value
        return settings


class Manager(manager.Manager):

    def __init__(self, driver):
        super().__init__(driver)
        self.create = Create(self)
        self.list = ListManager(self)
        self.create_user_capabilities_and_policies = \
            CreateUserCapabilitiesAndPolicies(self)
        self.create_admin_capabilities_and_policies = \
            CreateAdminCapabilitiesAndPolicies(self)
        self.get_roles = GetRoles(self)
        self.update_settings = UpdateSettings(self)
        self.remove_settings = RemoveSettings(self)
        self.get_application_settings_by_keys = \
            GetApplicationSettingsByKeys(self)



from viggocore.common import subsystem
from viggocore.subsystem.domain_sequence \
    import resource, controller, manager, router, driver


subsystem = subsystem.Subsystem(resource=resource.DomainSequence,
                                controller=controller.Controller,
                                manager=manager.Manager,
                                router=router.Router,
                                driver=driver.Driver)



from viggocore.common import subsystem
from viggocore.subsystem.tag import resource, router, manager, controller


subsystem = subsystem.Subsystem(resource=resource.Tag,
                                router=router.Router,
                                manager=manager.Manager,
                                controller=controller.Controller)



import flask
from typing import List

from viggocore.common.subsystem import controller
from viggocore.common import exception
from viggocore.common.input import InputResource, InputResourceUtils


class Controller(controller.Controller):

    def __init__(self, manager, resource_wrap, collection_wrap):
        super().__init__(manager, resource_wrap, collection_wrap)

    def _get_resources(self, input_resources) -> List[InputResource]:
        resources = []
        for input_resource in input_resources:
            endpoint = input_resource.get('endpoint')
            methods = input_resource.get('methods', [])
            resource = (endpoint, methods)
            resources.append(resource)

        return InputResourceUtils.parse_resources(resources)

    def create_policies(self, id: str):
        data = flask.request.get_json()
        try:
            resources = data.get('resources', [])
            data['resources'] = self._get_resources(resources)

            self.manager.create_policies(id=id, **data)

        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        return flask.Response(response=None,
                              status=204,
                              mimetype="application/json")



from viggocore.common import exception
from viggocore.subsystem.domain import tasks
from viggocore.common.subsystem import operation, manager
from viggocore.subsystem.image.resource import QualityImage
from viggocore.subsystem.domain.files import utils
from viggocore.common.utils import round_abnt


class DomainByName(operation.Operation):

    def pre(self, session, domain_name, **kwargs):
        self.domain_name = domain_name
        return True

    def do(self, session, **kwargs):
        domains = self.manager.list(name=self.domain_name)
        if not domains:
            raise exception.NotFound('ERROR! Domain name not found')
        domain = domains[0]

        # Hide user ID and settings from public resources
        domain.created_by = None
        domain.updated_by = None

        return domain


class DomainLogoByName(operation.Operation):

    def pre(self, session, domain_name, **kwargs):
        self.domain_name = domain_name
        return True

    def do(self, session, **kwargs):
        domains = self.manager.list(name=self.domain_name)
        if not domains:
            raise exception.NotFound('ERROR! Domain name not found')
        domain = domains[0]

        if domain.logo_id is None:
            raise exception.NotFound('ERROR! Domain logo not found')

        kwargs['quality'] = kwargs.get('quality', QualityImage.med)
        return self.manager.api.images().get(id=domain.logo_id, **kwargs)


class UploadLogo(operation.Update):

    def pre(self, session, id, token, file, **kwargs):
        self.file = file
        self.token = token

        return super().pre(session, id, **kwargs)

    def do(self, session, **kwargs):
        kwargs = {}
        kwargs['domain_id'] = self.entity.id
        kwargs['user_id'] = self.token.user_id
        kwargs['type_image'] = 'DomainLogo'

        image = self.manager.api.images().create(file=self.file, **kwargs)

        self.entity.logo_id = image.id

        return super().do(session=session)


class RemoveLogo(operation.Update):

    def do(self, session, **kwargs):
        logo_id = self.entity.logo_id
        self.entity.logo_id = None
        entity = super().do(session=session)
        if logo_id and entity is not None:
            self.manager.api.images().delete(id=logo_id)

        return


class Register(operation.Create):

    def _register_domain(self, domain_name, domain_display_name,
                         application_id, username, email, password):
        self.domain = self.manager.api.domains().create(
            application_id=application_id, name=domain_name,
            display_name=domain_display_name,
            addresses=[], contacts=[], active=False)

        self.user = self.manager.api.users().create(
            name=username, email=email, domain_id=self.domain.id, active=False)

        self.manager.api.users().reset(id=self.user.id, password=password)

    def pre(self, session, username, email, password,
            domain_name, domain_display_name, application_name):
        self.username = username
        self.email = email
        self.password = password
        self.domain_name = domain_name
        self.domain_display_name = domain_display_name
        self.application_name = application_name

        user_params_ok = username and email and password
        app_params_ok = domain_name and application_name
        if not user_params_ok and app_params_ok:
            raise exception.BadRequest(
                'ERROR! Not enough data to register domain')

        applications = \
            self.manager.api.applications().list(name=application_name)
        if not applications:
            raise exception.BadRequest('ERROR! Application name not found.')
        self.application = applications[0]

        return True

    def do(self, session, **kwargs):
        domains = self.manager.api.domains().list(name=self.domain_name)
        if not domains:
            self._register_domain(
                self.domain_name, self.domain_display_name,
                self.application.id, self.username, self.email, self.password)
        else:
            self.domain = domains[0]
            domain = domains[0]

            users = self.manager.api.users().list(email=self.email,
                                                  domain_id=domain.id)

            if domain.active:
                raise exception.BadRequest('Domain already activated')

            if not users or domain.display_name != self.domain_display_name:
                raise exception.BadRequest('Domain already registered')

            self.user = users[0]
            self.manager.api.users().reset(id=self.user.id,
                                           password=self.password)

        return True

    def post(self):
        # The notification don't be part of transaction must be on post
        tasks.send_email(self.user.id)


class Activate(operation.Create):

    def pre(self, session, token_id, domain_id, user_admin_id):
        if not (user_admin_id or domain_id):
            raise exception.BadRequest(
                'ERROR! Not enough data to Activate Domain')

        self.token_id = token_id
        self.domain_id = domain_id
        self.user_admin_id = user_admin_id

        roles = self.manager.api.roles().list(name='Admin')
        if not roles:
            raise exception.BadRequest('ERROR! Role Admin not found')
        self.role_admin = roles[0]

        return True

    def do(self, session, **kwargs):
        self.manager.api.domains().update(id=self.domain_id, active=True)
        self.manager.api.users().update(id=self.user_admin_id, active=True)
        self.manager.api.grants().create(user_id=self.user_admin_id,
                                         role_id=self.role_admin.id)
        self.manager.api.tokens().delete(id=self.token_id)

        domain = self.manager.api.domains().get(id=self.domain_id)
        if not domain:
            raise exception.BadRequest('ERROR! Domain not found')

        return domain


class UpdateSettings(operation.Update):

    def pre(self, session, id: str, **kwargs) -> bool:
        self.settings = kwargs
        if self.settings is None or not self.settings:
            raise exception.BadRequest("Erro! There is not a setting")
        return super().pre(session=session, id=id)

    def do(self, session, **kwargs):
        result = {}
        for key, value in self.settings.items():
            new_value = self.entity.update_setting(key, value)
            result[key] = new_value
        super().do(session)

        return result


class RemoveSettings(operation.Update):

    def pre(self, session, id: str, **kwargs) -> bool:
        self.keys = kwargs.get('keys', [])
        if not self.keys:
            raise exception.BadRequest('Erro! keys are empty')
        super().pre(session, id=id)

        return self.entity.is_stable()

    def do(self, session, **kwargs):
        result = {}
        for key in self.keys:
            value = self.entity.remove_setting(key)
            result[key] = value
        super().do(session=session)

        return result


class GetDomainSettingsByKeys(operation.Get):

    def pre(self, session, id, **kwargs):
        self.keys = kwargs.get('keys', [])
        if not self.keys:
            raise exception.BadRequest('Erro! keys are empty')
        return super().pre(session, id=id)

    def do(self, session, **kwargs):
        entity = super().do(session=session)
        settings = {}
        for key in self.keys:
            value = entity.settings.get(key, None)
            if value is not None:
                settings[key] = value
        return settings


class SendEmailActivateAccount(operation.Create):

    def pre(self, session, **kwargs):
        return True

    def do(self, session, **kwargs):
        domain_name = kwargs.get('domain_name', None)
        email = kwargs.get('email', None)
        domains = self.manager.api.domains().list(name=domain_name)
        if not domains:
            raise exception.NotFound('Domain not found')

        domain = domains[0]

        users = self.manager.api.users().list(email=email,
                                              domain_id=domain.id,
                                              name='admin')

        if domain.active:
            raise exception.BadRequest('Domain already activated')

        if not users:
            raise exception.NotFound('Invalid information')

        self.user = users[0]

        return True

    def post(self):
        # The notification don't be part of transaction must be on post
        tasks.send_email(self.user.id)


class GetFilesSize(operation.Get):

    def pre(self, session, id, **kwargs):
        return super().pre(session, id=id)

    def do(self, session, **kwargs):
        response_dict = utils.get_size(domain_id=self.id)
        return response_dict


class GetUsageInfoByDomain(operation.List):

    PRECISION = 2

    # vai pegar os totais de linhas em cada tabela do banco por domínio
    def _get_totals_by_domain(self, session):
        msg = """
            É necessário implementar a função get_totals_by_domain no
            projeto específico para garantir que as tabelas listadas e
            filtradas por domínio são apenas as importantes. O retorno
            deve ser uma lista de dicionários no seguinte formato:

            return = [
                {
                    'table_name': None,
                    'total': None,
                    'domain_name': None,
                    'application_id': None,
                    'application_name': None
                    'domain_id': None,
                    'active': None
                }
            ]
        """
        raise exception.BadRequest(msg)

    # vai pegar os totais de linhas em cada tabela do banco
    def _get_totals_db(self, session):
        msg = """
            É necessário implementar a função _get_totals_db no
            projeto específico para garantir que as tabelas listadas
            são apenas as importantes para aquele projeto. O retorno
            deve ser uma lista de dicionários no seguinte formato:

            return = {
                "entity_name1": total1,
                "entity_name2": total2,
                "entity_name3": total3
            }
        """
        raise exception.BadRequest(msg)

    def _normalizar_tamanho(self, tamanho: str):
        numero, unidade = tamanho.split(' ')
        numero = int(numero)
        if unidade == 'bytes':
            return numero
        elif unidade == 'kB':
            return (numero * 1024)
        elif unidade == 'MB':
            return (numero * 1024 * 1024)
        elif unidade == 'GB':
            return (numero * 1024 * 1024 * 1024)

    def _get_value_or_default(self, value, tipo):
        if value is not None:
            return value
        else:
            if tipo is str:
                return ''
            elif tipo is int:
                return 0

    # vai pegar o total de espaço de disco ocupado por cada tabela do banco
    def _get_disk_space_totals_by_db(self, session):
        from viggocore.subsystem.domain.sql.disk_space_totals_by_db \
            import QUERY
        rs = session.execute(QUERY)
        response = [r for r in rs]

        response_dict = {
            r[0]: self._normalizar_tamanho(r[1])
            for r in response}
        return response_dict

    # função responsável por pegar o objeto constante com as constantes para
    # realizar o cálculo
    def _get_const(self, application_id, table_name):
        for const in self.constants:
            if const.application_id == application_id and\
               const.table_name == table_name:
                return (const.p_processos, const.p_acessos_db)
        return (1, 1)

    # vai pegar as "constantes" que serão usadas para aplicar os pesos
    # de cada cálculo
    def _get_constant_for_calculation(self):
        self.constants, _ = self.manager.api.constant_for_calculations().list(
            active=True)

    def _apply_calculations(self):
        for i in range(0, len(self.totals_by_domain)):
            total = self.totals_by_domain[i].get('total', 0)
            table_name = self.totals_by_domain[i].get('table_name', '')
            # domain_name = self.totals_by_domain[i].get('domain_name', '')
            application_id = self.totals_by_domain[i].get(
                'application_id', '')
            percent = (total / self.totals_db.get(table_name, 1))
            disk_space_table = self.disk_space_totals_by_db_dict.get(
                table_name, 0)

            self.totals_by_domain[i]['percent'] = percent
            self.totals_by_domain[i]['total_space_disc_bank'] = (
                percent * disk_space_table)

            p_processos, p_acessos_db = self._get_const(
                application_id, table_name)
            self.totals_by_domain[i]['total_process'] = total * p_processos
            self.totals_by_domain[i]['total_acess_db'] = total * p_acessos_db

    def _make_base_dict(self, domain_name):
        return {
            'application_id': None,
            'application_name': None,
            'domain_id': None,
            'domain_name': domain_name,
            'total_entities': 0,
            'total_space_disc_bank': 0,
            'total_process': 0,
            'total_acess_db': 0,
            'total_space_disc_file': 0
        }

    def _group_by_domain(self):
        grand_totals_dict = {
            'total_entities': 0,
            'total_space_disc_bank': 0,
            'total_process': 0,
            'total_acess_db': 0,
            'total_space_disc_file': 0
        }

        domains = list(
            set(map(lambda x: x.get('domain_name', ''), self.totals_by_domain)))
        domains_dict = list(map(lambda x: self._make_base_dict(x), domains))
        response = []
        for dc in domains_dict:
            domain_name = dc.get('domain_name', '')
            list_aux = list(filter(lambda x: x['domain_name'] == domain_name,
                                   self.totals_by_domain))

            # preenche os campos do dicionário que estavam faltando
            dc['application_name'] = list_aux[0].get('application_name', None)
            dc['application_id'] = list_aux[0].get('application_id', None)
            dc['domain_id'] = list_aux[0].get('domain_id', None)
            dc['active'] = list_aux[0].get('active', False)

            # soma todos os totais de um domínio
            for aux in list_aux:
                total_entities = aux.get('total', 0)
                total_space_disc_bank = aux.get('total_space_disc_bank', 0)
                total_process = aux.get('total_process', 0)
                total_acess_db = aux.get('total_acess_db', 0)

                dc['total_entities'] += total_entities
                dc['total_space_disc_bank'] += total_space_disc_bank
                dc['total_process'] += total_process
                dc['total_acess_db'] += total_acess_db

            # calcula o espaço ocupado por arquivos no servidor para este
            # domínio
            dc['total_space_disc_file'] = self.manager.get_files_size(
                id=dc['domain_id']).get('BYTES', 0)

            # faz a somatória total de todos os campos
            grand_totals_dict['total_entities'] += dc['total_entities']
            grand_totals_dict['total_space_disc_bank'] += \
                dc['total_space_disc_bank']
            grand_totals_dict['total_process'] += dc['total_process']
            grand_totals_dict['total_acess_db'] += dc['total_acess_db']
            grand_totals_dict['total_space_disc_file'] += \
                dc['total_space_disc_file']

            response.append(dc)

        return (response, grand_totals_dict)

    def _calculate_cost_individual(self, usage, cost):
        return usage * float(cost)

    def _calculate_cost(self, usage):
        return ((usage * float(self.cost_servidor)) +
                (usage * float(self.cost_operacional)) +
                (usage * float(self.cost_administrativo)))

    def _calculate_usage(
            self, percent_total_entities, percent_total_space_disc_bank,
            percent_total_process, percent_total_access_db,
            percent_total_space_disc_file):
        usage_storage_bank = (
            float(percent_total_space_disc_bank) * self.p_storage_bank)
        usage_storage_file = (
            float(percent_total_space_disc_file) * self.p_storage_file)
        usage_process = float(percent_total_process) * self.p_process
        usage_access_db = float(percent_total_access_db) * self.p_access_db

        return (usage_storage_bank + usage_storage_file + usage_process +
                usage_access_db)

    def _normalize_values_response(self, response_item):
        response_item['total_space_disc_bank'] = round_abnt(
            response_item['total_space_disc_bank'], self.PRECISION)
        response_item['total_process'] = round_abnt(
            response_item['total_process'], self.PRECISION)
        response_item['total_acess_db'] = round_abnt(
            response_item['total_acess_db'], self.PRECISION)
        response_item['total_space_disc_file'] = round_abnt(
            response_item['total_space_disc_file'], self.PRECISION)
        response_item['percent_total_entities'] = round_abnt(
            response_item['percent_total_entities'], self.PRECISION)
        response_item['percent_total_space_disc_bank'] = round_abnt(
            response_item['percent_total_space_disc_bank'], self.PRECISION)
        response_item['percent_total_process'] = round_abnt(
            response_item['percent_total_process'], self.PRECISION)
        response_item['percent_total_access_db'] = round_abnt(
            response_item['percent_total_access_db'], self.PRECISION)
        response_item['percent_total_space_disc_file'] = round_abnt(
            response_item['percent_total_space_disc_file'], self.PRECISION)
        response_item['usage'] = round_abnt(
            response_item['usage'], self.PRECISION)
        response_item['cost_servidor'] = round_abnt(
            response_item['cost_servidor'], self.PRECISION)
        response_item['cost_operacional'] = round_abnt(
            response_item['cost_operacional'], self.PRECISION)
        response_item['cost_administrativo'] = round_abnt(
            response_item['cost_administrativo'], self.PRECISION)
        response_item['cost'] = (
            response_item['cost_servidor'] +
            response_item['cost_operacional'] +
            response_item['cost_administrativo']
        )
        response_item['cost'] = round_abnt(
            response_item['cost'], self.PRECISION)

        return response_item

    def _normalize_values_grand_totals_dict(self, grand_totals_dict):
        grand_totals_dict['total_entities'] = round_abnt(
            grand_totals_dict['total_entities'], self.PRECISION)
        grand_totals_dict['total_space_disc_bank'] = round_abnt(
            grand_totals_dict['total_space_disc_bank'], self.PRECISION)
        grand_totals_dict['total_process'] = round_abnt(
            grand_totals_dict['total_process'], self.PRECISION)
        grand_totals_dict['total_acess_db'] = round_abnt(
            grand_totals_dict['total_acess_db'], self.PRECISION)
        grand_totals_dict['total_space_disc_file'] = round_abnt(
            grand_totals_dict['total_space_disc_file'], self.PRECISION)
        grand_totals_dict['cost_servidor'] = round_abnt(
            grand_totals_dict['cost_servidor'], self.PRECISION)
        grand_totals_dict['cost_operacional'] = round_abnt(
            grand_totals_dict['cost_operacional'], self.PRECISION)
        grand_totals_dict['cost_administrativo'] = round_abnt(
            grand_totals_dict['cost_administrativo'], self.PRECISION)

        return grand_totals_dict

    def fill_in_percentages(self, response, grand_totals_dict):
        total_entities = grand_totals_dict['total_entities']
        total_space_disc_bank = grand_totals_dict['total_space_disc_bank']
        total_process = grand_totals_dict['total_process']
        total_acess_db = grand_totals_dict['total_acess_db']
        total_space_disc_file = grand_totals_dict['total_space_disc_file']

        for i in range(0, len(response)):
            response[i]['percent_total_entities'] = (
                response[i]['total_entities'] / total_entities)
            response[i]['percent_total_space_disc_bank'] = (
                response[i]['total_space_disc_bank'] / total_space_disc_bank)
            response[i]['percent_total_process'] = (
                response[i]['total_process'] / total_process)
            response[i]['percent_total_access_db'] = (
                response[i]['total_acess_db'] / total_acess_db)
            response[i]['percent_total_space_disc_file'] = (
                response[i]['total_space_disc_file'] / total_space_disc_file)

            response[i]['usage'] = self._calculate_usage(
                response[i]['percent_total_entities'],
                response[i]['percent_total_space_disc_bank'],
                response[i]['percent_total_process'],
                response[i]['percent_total_access_db'],
                response[i]['percent_total_space_disc_file']
            )
            usage = response[i]['usage']
            response[i]['cost_servidor'] = self._calculate_cost_individual(
                usage, self.cost_servidor)
            response[i]['cost_operacional'] = self._calculate_cost_individual(
                usage, self.cost_operacional)
            response[i]['cost_administrativo'] = self\
                ._calculate_cost_individual(usage, self.cost_administrativo)
            response[i]['cost'] = self._calculate_cost(response[i]['usage'])
            response[i] = self._normalize_values_response(response[i])

        return response

    def _valid_weights(self):
        total = (self.p_storage_bank + self.p_storage_file +  # noqa
                 self.p_process + self.p_access_db)
        if total > 1:
            raise exception.BadRequest(
                'the sum of the general weights must total 100%')

    def _get_cost_db(self, session, cost_name):
        return self.manager.api.project_costs().\
            get_cost_by_name_most_recent(
                session=session, name=cost_name, active=True)

    def _get_costs_params(self, **kwargs):
        session = kwargs.get('session', None)
        self.cost_servidor = kwargs.pop(
            'cost_servidor',
            self._get_cost_db(session, 'servidor'))
        self.cost_operacional = kwargs.pop(
            'cost_operacional',
            self._get_cost_db(session, 'operacional'))
        self.cost_administrativo = kwargs.pop(
            'cost_administrativo',
            self._get_cost_db(session, 'administrativo'))

        if self.cost_servidor is None:
            self.cost_servidor = 0
        if self.cost_operacional is None:
            self.cost_operacional = 0
        if self.cost_administrativo is None:
            self.cost_administrativo = 0

    def pre(self, **kwargs):
        self.de = kwargs.get('de', '2020-01-01')
        self.ate = kwargs.get('ate', '9020-01-01')
        self.domain_name = kwargs.get('domain_name', None)

        session = kwargs.get('session', None)

        self.p_storage_bank = float(kwargs.get('p_storage_bank', 0.1))
        self.p_storage_file = float(kwargs.get('p_storage_file', 0.1))
        self.p_process = float(kwargs.get('p_process', 0.3))
        self.p_access_db = float(kwargs.get('p_access_db', 0.5))

        self._valid_weights()

        precision = kwargs.get('precision', None)
        if precision is not None:
            try:
                self.PRECISION = int(precision)
            except Exception:
                raise exception.BadRequest(
                    'the precision parameter must be an integer')

        self.totals_db = self._get_totals_db(session)
        self.disk_space_totals_by_db_dict = self._get_disk_space_totals_by_db(
            session)
        self._get_constant_for_calculation()
        self.totals_by_domain = self._get_totals_by_domain(session)

        self._get_costs_params(**kwargs)
        return True

    def do(self, session, **kwargs):
        self._apply_calculations()

        response, grand_totals_dict = self._group_by_domain()
        response = self.fill_in_percentages(response, grand_totals_dict)

        grand_totals_dict['cost_servidor'] = self.cost_servidor
        grand_totals_dict['cost_operacional'] = self.cost_operacional
        grand_totals_dict['cost_administrativo'] = self.cost_administrativo
        grand_totals_dict['p_storage_bank'] = self.p_storage_bank
        grand_totals_dict['p_storage_file'] = self.p_storage_file
        grand_totals_dict['p_process'] = self.p_process
        grand_totals_dict['p_access_db'] = self.p_access_db
        grand_totals_dict = self._normalize_values_grand_totals_dict(
            grand_totals_dict)
        return (response, grand_totals_dict)


class Manager(manager.Manager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.domain_by_name = DomainByName(self)
        self.domain_logo_by_name = DomainLogoByName(self)
        self.upload_logo = UploadLogo(self)
        self.remove_logo = RemoveLogo(self)
        self.register = Register(self)
        self.activate = Activate(self)
        self.update_settings = UpdateSettings(self)
        self.remove_settings = RemoveSettings(self)
        self.get_domain_settings_by_keys = GetDomainSettingsByKeys(self)
        self.send_email_activate_account = SendEmailActivateAccount(self)
        self.get_files_size = GetFilesSize(self)
        self.get_usage_info_by_domain = GetUsageInfoByDomain(self)



# import flask_sqlalchemy
# db = flask_sqlalchemy.SQLAlchemy()
import flask_migrate

from flask_sqlalchemy import SQLAlchemy as _BaseSQLAlchemy


class SQLAlchemy(_BaseSQLAlchemy):
    def apply_pool_defaults(self, app, options):
        options = super().apply_pool_defaults(app, options)
        options["pool_pre_ping"] = True
        return options


db = SQLAlchemy()
migrate = flask_migrate.Migrate()



from viggocore.common.input import RouteResource
from typing import Any, Callable, Dict, List

from viggocore.common.subsystem import Subsystem
from viggocore.common.subsystem.manager import Manager
from viggocore.common.subsystem.transaction_manager import TransactionManager


class Api(object):

    def __init__(self, managers: Dict[str, Callable[[], Manager]],
                 bootstrap_resources: List[Any],
                 transaction_manager: TransactionManager) -> None:
        self.__instances: Dict[str, Manager] = dict()
        self.__bootstrap_resources = bootstrap_resources
        self.__transaction_manager = transaction_manager

        for name, fn in managers.items():
            setattr(self, name, self.__get_instance(name, fn))

    def __get_instance(self, name: str,
                       fn: Callable[[TransactionManager], Manager]) -> Callable[[TransactionManager], Manager]:
        def wrapper():
            manager = self.__instances.get(name)

            if not manager:
                manager = fn(self.__transaction_manager)
                setattr(manager, 'api', self)
                setattr(manager,
                        'bootstrap_resources',
                        self.__bootstrap_resources)
                self.__instances[name] = manager

            return manager

        return wrapper

    @property
    def transaction_manager(self):
        return self.__transaction_manager


class ApiHandler(object):

    def __init__(self, subsystems: Dict[str, Subsystem],
                 bootstrap_resources: Dict[str, RouteResource]) -> None:
        self.__managers_dict = {name: s.lazy_manager
                                for name, s in subsystems.items()}
        self.__bootstrap_resources = self.__get_resources(bootstrap_resources)

    def api(self, transaction_manager: TransactionManager = None) -> Api:
        tm = transaction_manager if transaction_manager is not None \
            else TransactionManager()
        return Api(self.__managers_dict, self.__bootstrap_resources, tm)

    def __get_resources(self, bootstrap_resources):
        def resources():
            None

        for key, resource in bootstrap_resources.items():
            setattr(resources, key, resource)

        return resources



from typing import List

from viggocore.common.subsystem import operation, manager
from viggocore.subsystem.route.resource import Route


class CreateRoutes(operation.Operation):

    def pre(self, routes: List[Route], session, **kwargs) -> bool:
        self.routes = routes
        return True

    def do(self, session, **kwargs) -> List[Route]:
        session.bulk_save_objects(self.routes)
        return self.routes


class Manager(manager.Manager):

    def __init__(self, driver):
        super().__init__(driver)
        self.create_routes = CreateRoutes(self)



from viggocore.common.subsystem import operation, manager


class GetCostByNameMostRecent(operation.List):

    def do(self, session, **kwargs):
        kwargs['order_by'] = 'created_at desc'
        costs = super().do(session, **kwargs)
        if len(costs) > 0:
            return costs[0].cost
        return None


class Manager(manager.Manager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.get_cost_by_name_most_recent = GetCostByNameMostRecent(self)



import os
from viggocore.common import exception


def get_size(domain_id):
    size = 0

    folderpath = os.environ.get('VIGGOCORE_FILE_DIR_SIZE', '')
    if len(folderpath) == 0:
        raise exception.BadRequest('VIGGOCORE_FILE_DIR_SIZE not found')

    for ele in os.scandir(folderpath):
        if '.' not in ele.name:
            for ele2 in os.scandir(f'{folderpath}/{ele.name}'):
                if '.' not in ele2.name:
                    for ele3 in os.scandir(f'{folderpath}/{ele.name}/{ele2.name}'):
                        if ele3.name == domain_id:
                            for ele4 in os.scandir(f'{folderpath}/{ele.name}/{ele2.name}/{ele3.name}'):
                                for ele5 in os.scandir(f'{folderpath}/{ele.name}/{ele2.name}/{ele3.name}/{ele4.name}'):
                                    size += os.path.getsize(ele5)

    response_dict = {}

    # montandos as respostas
    conversoes = ['BYTES', 'KBYTES', 'MEGAS', 'GIGAS', 'TERAS']
    for conversao in conversoes:
        response_dict.update({conversao: size})
        size /= 1024

    return response_dict



import json

import flask

from viggocore.common import exception
from viggocore.common.exception import BadRequest
from viggocore.common.subsystem import controller

# TODO(samueldmq): take a better look at this, it is completely different as it
# is not dealing with JSON content


def get_bad_request(cause=None):
    bad_request = BadRequest()
    bad_request.message = cause if not None else bad_request.message

    return flask.Response(response=bad_request.message,
                          status=bad_request.status)


class Controller(controller.Controller):

    def get_token_id(self):
        return flask.request.headers.get('token')

    def get_token(self, token_id):
        return self.manager.api.tokens().get(id=token_id)

    def get_domain(self, domain_id):
        return self.manager.api.domains().get(id=domain_id)

    def get_domain_id_from_token(self, token):
        user = self.manager.api.users().get(id=token.user_id)
        return user.domain_id

    def get_domain_id(self):
        token = self.get_token(self.get_token_id())
        domain_id = self.get_domain_id_from_token(token)
        return domain_id

    def create(self, **kwargs):
        # TODO(samueldmq): the file should be extracted here.
        # the todo above would be resolved too with this!
        try:
            file = flask.request.files.get('file', None)
            if not file:
                return get_bad_request('file é obrigatório')

            token = self.get_token(self.get_token_id())
            domain_id = self.get_domain_id_from_token(token)
            if not domain_id:
                return get_bad_request(
                    'Não foi possível determinar o domain_id')

            user_id = token.user_id
            if not user_id:
                return get_bad_request(
                    'Não foi possível determinar o user')

            kwargs['domain_id'] = domain_id
            kwargs['user_id'] = user_id
            entity = self.manager.create(file=file, **kwargs)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        response = {self.resource_wrap: entity.to_dict()}

        return flask.Response(response=json.dumps(response, default=str),
                              status=201,
                              mimetype="application/json")

    def get(self, id):
        return self.manager.get(id=id)
